@@ -1,4 +1,4 @@
-#!/usr/bin/env python3.7
+#!/usr/bin/env python3
 """
     A collection of reused functions and classes.
     Depends only on python standard library.
@@ -11,7 +11,7 @@ import asyncio
 import inspect
 import logging
 from time import time, sleep
-from uuid import uuid4
+from types import ModuleType
 from pathlib import Path
 from datetime import datetime, date, timezone
 from functools import wraps
@@ -55,11 +55,17 @@ def utcnowtz(): return datetime.now(tz=timezone.utc)
 
 
 def isoformat(datetime_instance, timespec='auto'):
+    kwargs = {}
+    if isinstance(datetime_instance, datetime):
+        # don't pass timespec if type is not date not datetime
+        kwargs['timespec'] = timespec
+
     return (datetime_instance
-            .isoformat(timespec=timespec)
+            .isoformat(**kwargs)
             .replace('.', ',')
             .replace('+00:00', 'Z'))
 
+
 def isoformat_safe(datetime_instance, timespec='auto'):
     """ portable file system safe iso format (sigh) """
     # FIXME this is super inefficient implement from scratch
@@ -189,13 +195,19 @@ def stack_magic(stack):
     # REMEMBER KIDS ALWAYS CALL inspect.stack(0) if you don't want
     # to acess the file system for every frame! (still slow, but better)
     function_names = [f.function for f in stack]
-    if len(stack) > 2 and 'exec_module' in function_names:
+    if len(stack) > 2 and ('exec_module' in function_names or
+                           '_run_module_as_main' in function_names):
         for i, frame in enumerate(stack):
             fl = frame[0].f_locals
+            if 'import_module' in fl:
+                for v in fl.values():
+                    if type(v) is ModuleType and hasattr(v, '__globals__'):
+                        return v.__globals__
+
             if '__builtins__' in fl:
                 #print('globals found at', i)
                 return fl
-    elif in_notebook or in_ipython or in_test or '_run_module_as_main' in function_names:
+    elif in_notebook or in_ipython or in_test:
         index = 1  # this seems to work for now
     elif function_names.count('main') >= 2:
         # FIXME this is a hack that only works
@@ -225,6 +237,21 @@ def subclasses(start):
             yield from subclasses(sc)
 
 
+def subclass_tree(start, node_type=tuple):
+    if issubclass(start, type):
+        scs = start.__subclasses__(start)
+    else:
+        scs = start.__subclasses__()
+
+    for sc in scs:
+        if sc is not None:
+            subclasses = node_type(subclass_tree(sc))
+            if subclasses:
+                yield node_type((sc, *subclasses))
+            else:
+                yield node_type((sc,))
+
+
 def getSourceLine(cls):
     tc = TermColors
     try:
@@ -964,5 +991,8 @@ def asStr(astNode, prior=tuple(), dereference=False):
             return astNode.name + ((' as ' + astNode.asname)
                                    if astNode.asname else
                                    '')
+    elif isinstance(astNode, ast.Constant):
+        # e.g. bools True and False
+        return str(astNode.value)
     else:
         raise NotImplementedError(f'{astNode}')
